//
//  ObjectDynamic.m
//  Creatures
//
//  Created by Lenka on 07/01/2010.
//  Copyright 2010 Lenka Pitonakova. All rights reserved.
//

#import "DynObjController.h"
#import "DynObjView.h"
#import "DynObjMemory.h"
#import "World.h"
#import "DynObjModel.h"


@implementation DynObjController

@synthesize m_pWorld;
@synthesize m_iType;

- (id) initDynObj: (Byte)id_ type:(Byte)type_ pos:(CGPoint)pos_ rot:(CGFloat)rot_ world:(World*)world_{
	[super initStandard:id_ pos:pos_ size:CGPointMake(40,40) rot:rot_ txtrId:TEX_OBJS_DYN];
	m_pWorld = world_;
	m_iType = type_;
	m_pView = [[DynObjView alloc] initViewWithController:self];	
	m_pMemory = [[DynObjMemory alloc] initMemoryWithController:self ];	
	m_iToIdleCount = -1;
	m_ptUcoordins = CGPointMake(0,0.5);
	m_ptVcoordins = CGPointMake(0,1);
	return self;
}


- (void) dealloc {
	[m_pView release];
	[m_pMemory release];
	[super dealloc];
}
- (void) Update {
	//-------------- determines whether to render:
	[super Update];
	//--------------------- test to idle count, descent it if scheduled for idle state and go idle when count reaches 0:
	if (m_iToIdleCount >= 0) {
		m_iToIdleCount --;
		if (m_iToIdleCount < 0) {
			[self GoIdle];
		}
	}
}

- (void) DoRendering {
	[m_pView RenderAt:m_ptPos rot:m_fRot txtrId:m_iTextureId uCoordins:m_ptUcoordins vCoordins:m_ptVcoordins];
}

#pragma mark ================================ INTERFACE FOR MODEL CLASSES

- (BOOL) GetIsMoving {
	if ([m_pMemory GetCurTaskName] == UORDER_MOVE) {
		return true;
	} else {
		return false;
	}
}


#pragma mark ================================ USER INTERACTION

- (void) GoIdle{
	if ([m_pMemory GetCurTaskName] != UORDER_MOVE) {
		[self UserOrder:UORDER_NONE	atLocation:CGPointMake(0,0) withObject:NULL];
		[self Unselect];
	}
}
- (void) ScheduleIdleStateIn:(NSInteger)units_ {
	m_iToIdleCount = units_;
}

- (void) UserOrder:(Byte)order_ atLocation:(CGPoint)loc_ withObject:(DynObjController*)withObj_ {
	[m_pMemory SetTaskTo:order_ atLocation:loc_ withObject:withObj_];
}

- (void) FinishedTask:(Byte)task_ {
	[self ScheduleIdleStateIn:SECONDS_UNTIL_IDLE*FPS];
	[m_pMemory SetTaskTo:UORDER_NONE atLocation:CGPointMake(0,0) withObject:NULL];
}

- (void) Unselect {
	g_bSelectedUnits[m_iId] = false;
	m_pView.m_bSelected = false;
}


#pragma mark ================================= TOUCHES
- (void) HandleTouchesBeginAt:(CGPoint)pos_ {	
	[m_pView HandleTouchesBeginAt:pos_ selfPos:m_ptPos selfSize:m_ptSize];	
}

- (void) HandleTouchesMovedAt:(CGPoint)pos_ {	
	[m_pView HandleTouchesMovedAt:pos_ selfPos:m_ptPos selfSize:m_ptSize];	
}

- (void) HandleTouchesEndedAt:(CGPoint)loc_ {
	[m_pView HandleTouchesEndedAt:loc_ selfPos:m_ptPos selfSize:m_ptSize];
	
}

#pragma mark ================================== ENCODING
/* 
 The function returns path to a data file for a specified object key in form objectKey_controllerId.
 The '_controllerId' part of the file path makes sure no two objects get encoded into the same file.
 */
- (NSString *) dataFilePathForKey: (NSString*)key_ {
	//--- look for the Documents directory within the application's sandbox
	NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
	NSString *documentsDir = [paths objectAtIndex:0];
	NSString *fileName =[[NSString alloc] initWithFormat:@"%@_%d",key_,m_iId];
	//------- append the filename to get its full path:
	return [documentsDir  stringByAppendingPathComponent:fileName];
	[fileName release];
	[paths release];
	[documentsDir release];
}

- (void) DecodeModelObject:(DynObjModel*) modelObject_ usingKey:(NSString*) objectKey_{
	NSString *filePath = [self dataFilePathForKey:objectKey_];
	//------------- if file exists, copy its content into data object and create a new instance:
	if ([[NSFileManager defaultManager] fileExistsAtPath:filePath]) {
		NSData *data = [[NSMutableData alloc] initWithContentsOfFile:[self dataFilePathForKey:objectKey_]];
		NSKeyedUnarchiver *unarchiver = [[NSKeyedUnarchiver alloc] initForReadingWithData:data];
		DynObjModel * Model = [unarchiver decodeObjectForKey:objectKey_];
		[modelObject_ SetSelfWith:Model];
		[unarchiver finishDecoding];		
		[unarchiver release];
		[data release];
	} 
}

- (void)EncodeModelObject:(DynObjModel*)modelObject_ usingKey:(NSString*)objectKey_ {
	NSMutableData *data = [[NSMutableData alloc] init];
	NSKeyedArchiver *archiver = [[NSKeyedArchiver alloc] initForWritingWithMutableData:data];
	[archiver encodeObject:modelObject_ forKey: objectKey_];
	[archiver finishEncoding];
	[data writeToFile:[self dataFilePathForKey:objectKey_] atomically:YES];
	//---------
	[archiver release];
	[data release];
	
}


@end
