//
//  AgentModel.m
//  Creatures
//
//  Created by Lenka on 23/12/2009.
//  Copyright 2009 Lenka Pitonakova. All rights reserved.
//

#import "DynObjModel.h"
#import "DynObjController.h"


@implementation DynObjModel

@synthesize m_pController;

- (id) initModelWithController:(DynObjController*)controller_ encodingKey:(NSString*)key_{
	[super init];
	m_pController = controller_;
	m_sEncodingKey = key_;
	//----------------- call AgentController's universal method for Model decoding. AgentController determines whether encoded data for specific class exist.
	[m_pController DecodeModelObject: self usingKey: key_];
	
	//--------- get reference to the application instance
	UIApplication *app = [UIApplication sharedApplication];
	
	//---------- subscribe to the application terminate notification, using this class as observer (implements a method that handles it specified as selector. Name of the notification is a keyword. Object specifies which object notifies.
	[[NSNotificationCenter defaultCenter] addObserver:self
											 selector:@selector(applicationWillTerminate:)
												 name:UIApplicationWillTerminateNotification 
											   object:app];	
	return self;
}

- (void) applicationWillTerminate: (NSNotification *) notification {
	[m_pController EncodeModelObject:self usingKey:m_sEncodingKey];
}
#pragma mark ======================================= NSCoding
/*
 Encode and decode class variables of individual Model objects.
 Only variables which change their value during the program runs need to be encoded/decoded.
*/


/*
 encodeWithCoder:
 - encodes an object when application closes
*/
- (void) encodeWithCoder: (NSCoder*) encoder {	
	/* to encode a varialbe use e.g:
	 [encoder encodeInteger: m_iMyInt forKey: k1];
	 - individual keys are supplied in the header of this class.
	 */
}

/* initWithCoder:
 - decodes and object when application starts
*/
- (id) initWithCoder: (NSCoder *) decoder {
	
	if (self == [super init]) {
		/* decode variables listed in encodeWithCoder: e.g.
		 m_iMyInt = [decoder decodeIntegerForKey:k1];
		 */
	}
	return self;
}
/* SetSelfWith:
 - copies properties over from a decoded object
 - !!! set the class of decodedObj_ to the class which implements the method
 */
- (void) SetSelfWith: (DynObjModel*)decodedObj_  {
	/* copy properties listed in encodeWithCoder: e.g
	m_iMyInt = decodedObj_.m_iMyInt;
	 */
}




@end
